#' Identifying Cell Populations in Flow Cytometry Data Using Phenotypic Signatures
#'
#' This fucntion identify homoginious cell populations within the cytometry data.
#'
#' @param x a N x M matrix of N cells including M cell markers
#' @return a vector of generated clustered labels
#' @author Maziyar Baran Pouyan
#' @details
#' This method works based on a hybrid technique that estimates the initial clusters in high dimension
#' and identifies the final clusters by merging clusters using their phenotypic signatures in low dimension.
#' @seealso \code{Rtsne}
#' @export
#' @importFrom Rtsne Rtsne
#' @importFrom feature featureSignif
#' @importFrom apcluster apcluster
#' @importFrom pbapply pblapply
#' @importFrom cluster clara
#' @examples
#' library(SigClust)
#' data(x,package = "SigClust")
#' Label<-SigClust(x)


SigClust<-function(x){

  #   source("C:/Users/mxb112230/Documents/SigClust/SigClust/R/FindSubClouds.R")
  #
  #   source("C:/Users/mxb112230/Documents/SigClust/SigClust/R/Merger.R")
  #
  #   source("C:/Users/mxb112230/Documents/SigClust/SigClust/R/ModeCount.R")
  #
  #   source("C:/Users/mxb112230/Documents/SigClust/SigClust/R/Normalization.R")
  #
  #   source("C:/Users/mxb112230/Documents/SigClust/SigClust/R/PeakFinding.R")
  #
  #   source("C:/Users/mxb112230/Documents/SigClust/SigClust/R/SigExtraction.R")
  #
  #   source("C:/Users/mxb112230/Documents/SigClust/SigClust/R/TwoDTsneEmbedding.R")


  data1<-x


  MainData<-sapply(as.data.frame(data1),normal)
  #MainData <- scale(data1, center=T, scale=T)


  k<-ModeCount(MainData)


  clarares<-clara(MainData,k,100)


  Initiallabeling<-clarares$clustering


  ns<-clarares$i.med


  InitialCenters<-(MainData[ns,])


  Kstar<-FindSubClouds(InitialCenters)


  Sigs<-SigExtraction(MainData,k,Initiallabeling)


  EmbeddedSigs<-TwoDTsneEmbedding(Sigs)


  finalpeaklabel<-PeakFinding(EmbeddedSigs,Kstar)


  Finallabeling<-Merger(MainData,finalpeaklabel,Initiallabeling)

  cluster_data<-data1
  cluster_data$cluster_label<-Finallabeling
  write.csv(cluster_data, file="Clustered_Data.csv") #Optionally write embedding to a csv-file

  return(Finallabeling)

}


#' Identifying Cell Populations in Flow Cytometry Data Using Phenotypic Signatures
#'
#' This fucntion identify homoginious cell populations within the cytometry data.
#'
#' @param x a N x M matrix of N cells including M cell markers
#' @param y is vector including cluster labels computed by SigClust function
#' @return a 2D t-SNE plot visualizing identified populations
#' @author Maziyar Baran Pouyan
#' @details
#' This function visualizes identified populations on a 2D plot using t-SNE technique
#' @export
#' @importFrom Rtsne Rtsne
#' @importFrom ggplot2 ggplot
#' @importFrom ggplot2 aes
#' @importFrom ggplot2 geom_point
#' @importFrom ggplot2 scale_color_manual
#' @importFrom ggplot2 theme
#' @importFrom ggplot2 guides
#' @importFrom ggplot2 guide_legend
#' @examples
#' tsne2dplot(x,Label)



tsne2dplot<-function(x,y){

  labelcoloring<-y

  MainData <- scale(x, center=T, scale=T)

  cat("Computing the embeddings of dataset: ", "\n")


  tsne_out <- Rtsne(as.matrix(MainData), dims = 2, initial_dims = 8, perplexity = 30, theta = 0.5,
                    check_duplicates = FALSE, pca = FALSE,verbose=TRUE)

  data2D<-tsne_out$Y



  cluster<-factor(labelcoloring)

  ggplot(as.data.frame(data2D),aes(x=V1,y=V2,colour=cluster))+geom_point()+theme(legend.position="none")+scale_color_manual(values=c("dodgerblue4", "darkolivegreen4",
                                                                                                                                     "darkorchid3", "goldenrod1","darkgreen","darkorange","deeppink","chartreuse","lightcoral","dodgerblue","orangered","navajowhite3","gray8","yellow","darkslateblue","red","red4","deepskyblue4","orangered2","green","lightcyan2","indianred3","magenta4","plum","seagreen","yellow4","green4","red4"))+
    guides(col=guide_legend(nrow=7))

}


#' Identifying Cell Populations in Flow Cytometry Data Using Phenotypic Signatures
#'
#' This fucntion identify homoginious cell populations within the cytometry data.
#'
#' @param x a N x M matrix of N cells including M cell markers
#' @param y is a vector including cluster labels computed by SigClust function
#' @return Cell population phenotypic patterns
#' @author Maziyar Baran Pouyan
#' @details
#' This function calculates and stores one phenotypic pattern for each identified cell population.
#' @export
#' @importFrom ggplot2 ggsave
#' @importFrom ggplot2 position_dodge
#' @importFrom ggplot2 geom_bar
#' @importFrom ggplot2 element_text
#' @importFrom ggplot2 ggtitle
#' @importFrom ggplot2 geom_errorbar
#' @importFrom ggplot2 geom_crossbar
#' @importFrom ggplot2 xlab
#' @importFrom ggplot2 ylab
#' @examples
#' PhynoSigPattern(x,Label)
PhynoSigPattern <- function(x,y){

  data1<-x
  Finallabeling<-y
  subDir <- "PhenPattern"

  if (! file.exists(subDir)){
    dir.create(subDir)
  }

  setwd(subDir)

  #ggsave(filename = "Clustered_Data.pdf")

  ################################Phyno_Patterns


  ####This part is extracted from ACCENSSE tool to generate the bar plots##

  med_all <- data.frame(marker = names(data1), median_exp = apply(data1,2,median))


  for (i in 1:max(Finallabeling)){

    graphics.off()

    test <- data1[which(Finallabeling==i),]
    #test<-MainData[L[,3],]


    cat(i,"\n")

    pop_median <- data.frame(marker = names(test), median_exp = apply(test, 2, median), row.names = NULL)
    pop_mad <- apply(test, 2, mad)
    limits <- aes(ymax = pop_median$median_exp + pop_mad/2, ymin = pop_median$median_exp - pop_mad/2)
    dodge <- position_dodge(width = 0.7)

    #bar graphs
    fname1 = paste('Subpop',as.character(i),'.pdf',sep="")

    title = paste("Subpopulation ", as.character(i))


    ggplot(pop_median,aes(x=marker,y=median_exp)) +
      geom_bar(width = 0.7, position = dodge, stat="identity", color = "black", fill = "green") +
      theme(axis.text.x = element_text(angle=45, hjust=1)) +
      ggtitle(title) + xlab("Cellular markers") + ylab("Expression level") +
      geom_errorbar(limits,position = dodge, width=0.3) +
      geom_crossbar(data = med_all, aes(marker, median_exp, ymin = median_exp, ymax = median_exp),color = "blue")

    #Save file

    ggsave(filename = fname1)

  }

  graphics.off()

  setwd('..')

}

#' BiaxPlots
#'
#' This fucntion generate all the possible 2D plts of cell markers.
#'
#' @param x a N x M matrix of N cells including M cell markers
#' @param y is a vector including cluster labels computed by SigClust function
#' @return All the possible biaxial plots of cell markers
#' @author Maziyar Baran Pouyan
#' @details
#' This function visualize all the identified cell populations on all the possible biaxial plots of cell markers.
#' @export
#' @examples
#' library(SigClust)
#' data(x,package = "SigClust")
#' Label<-SigClust(x)
#' BiaxPlots(x,col=Label,pch=20)

BiaxPlots <- function(x,y){

  plot(x,col=y,pch=20)

}



